/*****************************************************************************
 * $LastChangedDate: 2009-05-18 13:53:36 -0500 (Mon, 18 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Main header file.
 * @namespace base
 * @brief Library of base code.
 *//*
 * LEGAL:	COPYRIGHT (C) 2004 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef BASE_MODULE_HH
#define BASE_MODULE_HH 1

#include <string>
using namespace std;

//------------------------------------------------------------------------------
// Coding convention for modules:
// - module.hh is the main header files that includes often-used header files.
// - Init() initializes the module (if applicable).
// - Start() starts the execution of a module (if applicable).

namespace base {

////////////////////////////////////////////////////////////////////////////////
/// @brief InitArgs struct is passed to Init() of every module.
///
struct InitArgs
{
	InitArgs( int argc, char** argv, const string& programName )
	: mArgc(argc), mArgv(argv), mProgramName(programName)
	{
	}

	int		mArgc;
	char**	mArgv;
	string	mProgramName;
};

void Init( const InitArgs& initArgs );

void Start( void );

} // namespace base

//------------------------------------------------------------------------------

// Instead of writing "static" which raises concerns about thread-safety.
#define PERSISTENT   static
#define INLINE       static inline
#ifndef INTERN					// tests can predefine to test static functions
#define INTERN       static
#endif
#ifndef INTERNAL
#define INTERNAL     static
#endif
#define CLASS_CONST  static const
#define CLASS_VAR    static
#define CLASS_METHOD static

/// (Historical: used to have per-thread streams.)
#define COUT   cout
#define CERROR cerr
#define CLOG   clog
#define CDEBUG clog

// To declare globals in header files.
#if BASE_MODULE_CC
#define BASE_GLOBAL  // instantiate when .cc is compiled
#else
#define BASE_GLOBAL extern
#endif

#ifdef OS_WINDOWS
#undef uint
#define uint unsigned int
#endif

// Include only usual headers.
#if DEBUG
#include "base/typesig_defs.hh"
#endif
#include "base/typesig.hh"
#include "base/sys.hh"
#include "base/cc.hh"
#include "base/exception.hh"
#include "base/assert.hh"
#include "base/defs.hh"   // general
#include "base/defs2.hh"  // specific
#include "base/debug.hh"
#include "base/macros.hh"
#include "base/funcs.hh"
#include "base/thread_single.hh"  // vestige
#include "base/types.hh"
#include "base/safe_ptr.hh"
#include "base/shared_ptr.hh"
#include "base/globals.hh"

#endif // BASE_MODULE_HH
