/*****************************************************************************
 * $LastChangedDate: 2009-05-15 14:25:58 -0500 (Fri, 15 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Texture class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef FX_TEXTURE_HH
#define FX_TEXTURE_HH 1

#include <osg/Texture2D>

namespace fx {

namespace defs {
// Texture unit assignments:
const int TEXTURE_UNIT_0        = 0;  // should be int for GLSL uniforms
const int TEXTURE_UNIT_SHADOW   = 3;
}  // namespace defs

////////////////////////////////////////////////////////////////////////////////
/// @brief Texture class abstracts a texture object, and loads/caches texture image files.
///
/// Texture ctor uses ImageCache.
///
class Texture : public Shared
{
PREVENT_COPYING(Texture)
public:
				Texture( const string& filename );
        		~Texture();
	void		Bind( osg::StateSet& stateset );  // difficult to abstract osg::StateSet
	CLASS_METHOD RefPtr<osg::StateSet> NewTextureAsStateSet( const string& filename );

private:
	RefPtr<osg::Texture2D>	mTextureObject;
public:
	DECL_TYPESIG(TYPESIG_TEXTURE)
};

} // namespace fx

#endif // FX_TEXTURE_HH
