/*****************************************************************************
 * $LastChangedDate: 2009-05-16 16:27:08 -0500 (Sat, 16 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	OSG utility functions.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define GFX_OSG_FUNCS_CC 1
#include "base/module.hh"
using namespace base;
#include "gfx/module.hh"
#include "gfx/osg.hh"

namespace gfx {

/*****************************************************************************
 * Remove all child nodes.
 *****************************************************************************/
void
OsgRemoveChildren( RefPtr<osg::Group> group )
{
    const uint n = group->getNumChildren();
    group->removeChildren( 0, n );
}

/*****************************************************************************
 * Call this after modifying elements vertexs (while keeping the same size).
 *****************************************************************************/
void
OsgGeometryWasModified( RefPtr<osg::Geometry> geom,
                        RefPtr<osg::Vec3Array> vertexs )
{
	// Update affected components of osg::Geometry.
	geom->setVertexArray( vertexs.get() );		// recomputes bounding box, calls dirtyDisplayList()
}

/*****************************************************************************
 * Call this after adding or removing vertexs.
 * NOTE: Assumes osg::Geometry has one PrimitiveSet.
 *****************************************************************************/
void
OsgGeometryWasModifiedByResizing( RefPtr<osg::Geometry> geom,
                                  RefPtr<osg::Vec3Array> vertexs,
                                  RefPtr<osg::DrawArrays> drawArray )
{
	// Update affected components of osg::Geometry.
	geom->setVertexArray( vertexs.get() );		// recomputes bounding box, calls dirtyDisplayList()
    drawArray->setCount( vertexs->size() );		// updates glDrawArray() parameters (OpenGL vertex array)
}

} // namespace gfx
