/*****************************************************************************
 * $LastChangedDate: 2009-05-16 16:27:08 -0500 (Sat, 16 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Base scene-graph class.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef GRAPH_BASE_SCENE_GRAPH_HH
#define GRAPH_BASE_SCENE_GRAPH_HH 1

#include "object/module.hh"
#include "object/events.hh"
namespace object { class Object; }
using namespace object;
namespace graph { class NodeSort; }

namespace graph {

////////////////////////////////////////////////////////////////////////////////
/// @brief Base scene-graph class.
///
/// See scene_graph.hh.
///
/// Functionality of this base class:
/// - creates and caches "branch nodes" for state-sorting according to NodeSort
/// - can attach low-level nodes
/// - can attach higher-level Objects
/// - automatically detaches Objects when DestroyObject event is broadcast
///
class BaseSceneGraph : public osg::Referenced
{
PREVENT_COPYING(BaseSceneGraph)
protected:
                BaseSceneGraph( RefPtr<GroupNode> rootNode = new GroupNode );
    virtual     ~BaseSceneGraph() = 0;

// Public interface:
public:
    virtual RefPtr<GroupNode>   GetRootNode( void ) { return mBaseRootNode.get(); }
    virtual void                AttachNode( RefPtr<Node> node, const NodeSort& nodeSort );
    void                        AttachBranchNode( RefPtr<GroupNode> branchNode );
    virtual void                AttachObject( shptr<Object> object, const NodeSort& nodeSort );
    virtual void                DetachNode( RefPtr<Node> node );
    void                        DetachBranchNode( RefPtr<GroupNode> branchNode );
    virtual void                DetachObject( shptr<Object> );

// Private/protected methods:
protected:
    void                        SetRootNode( RefPtr<GroupNode> rootNode );
    void                        SetDefaultState( RefPtr<Node> node );
    void                        AttachNode( RefPtr<Node> node, RefPtr<GroupNode> parent );
    void                        Optimize( RefPtr<Node> node, const NodeSort& nodeSort );
private:
    CLASS_DESTROY_OBJECT_LISTENER( BaseSceneGraph, mBaseSceneGraph )

// Data members:
private:
    typedef map<NodeSort,RefPtr<GroupNode> > NodeSortMap;
    typedef set<shptr<Object> > Objects;
    CLASS_VAR NodeSort      msBranchNodeSort;	///< default branch has default NodeSort
    RefPtr<GroupNode>       mBaseRootNode;     	///< the absolute root node
    RefPtr<GroupNode>       mBranchNode;       	///< group node with default state
    NodeSortMap             mNodeSortMap;      	///< sorts nodes with shared state
    Objects                 mObjects;       	///< AttachObject() implies hold ref to Object
};

} // namespace graph

#endif // GRAPH_BASE_SCENE_GRAPH_HH
