/*****************************************************************************
 * $LastChangedDate: 2009-02-08 09:49:09 -0600 (Sun, 08 Feb 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	OSG-specific Keyboard class and special trackball keyboard handler.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#if COMPILE_OSG
#ifndef INPUT_KEYBOARD_OSG_HH
#define INPUT_KEYBOARD_OSG_HH 1

#undef KEY_F1  // workaround linux/input.h
#undef KEY_F2 
#undef KEY_F3
#undef KEY_F4
#undef KEY_F5
#undef KEY_F6
#undef KEY_F7
#undef KEY_F8
#undef KEY_F9
#undef KEY_F10
#undef KEY_F11
#undef KEY_F12
#undef KEY_F13

#include <osgGA/GUIEventHandler>

namespace input {

////////////////////////////////////////////////////////////////////////////////
/// @brief Keyboard class specialized for OSG.
///
class KeyboardOSG : public Keyboard
{
PREVENT_COPYING(KeyboardOSG)
friend class Keyboard;  // for Keyboard::GetInstance()
private:
    // Keyboard is meant to be instantiated once.
	KeyboardOSG( void );
	~KeyboardOSG();

private:
    /// @brief Handles keyboard events (OSG).
    class HandlerOSG : public osgGA::GUIEventHandler
    {
    public:
    					HandlerOSG( void ) { }
    					~HandlerOSG() { }
        virtual bool	handle( const osgGA::GUIEventAdapter& event, osgGA::GUIActionAdapter& );
	    virtual void	accept( osgGA::GUIEventHandlerVisitor& visitor );
    };

public:
	void				Enable( void );
};

////////////////////////////////////////////////////////////////////////////////
/// @brief Minimal keyboard handler for trackball mode.
///
/// Window installs a minimal keyboard handler since the Lua scripts
/// (which normally handle keyboard events) don't fully run in trackball mode.
///
class TrackballKeyboardHandler : public osgGA::GUIEventHandler
{
public:
    virtual bool handle( const osgGA::GUIEventAdapter& event, osgGA::GUIActionAdapter& );
};

} // namespace input

#endif // INPUT_KEYBOARD_OSG_HH
#endif // COMPILE_OSG
