/*****************************************************************************
 * $LastChangedDate: 2009-05-20 15:51:18 -0500 (Wed, 20 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Initialize/start module.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define INPUT_MODULE_CC 1
#include "base/module.hh"
using namespace base;
#include "glue/module.hh"
using namespace glue;
#include "input/module.hh"
#include "input/conf.hh"

namespace input {

/*****************************************************************************
 * Initialize module.
 *****************************************************************************/
void
Init( const base::InitArgs& initArgs )
{
	// Load textual conf file into Conf struct.
	const string confTxtFilename = GetConfTxtFilename( initArgs.mProgramName );
	// Not CONF macro since mValid isn't true yet.
	input::global::gConf.Init( initArgs.mArgc, initArgs.mArgv, confTxtFilename );
}

/*****************************************************************************
 * Start module.
 * @pre Higher-level code has called Joystick::Enable() if appropriate.
 *****************************************************************************/
void
Start( void )
{
	// Enable input queue.
    // Assumes that client called Joystick::Enable() prior.
	InputQueue::GetInstance().Enable();

    // Joystick calibration (and even enablement) is deferred up
    // to the higher level "control" module.
}

} // namespace input
