/*****************************************************************************
 * $LastChangedDate: 2009-06-15 11:51:43 -0500 (Mon, 15 Jun 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "physics/module.hh"
#include "physics/physics_control.hh"
using namespace physics;
#include "program/module.hh"
using namespace program;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Program"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * string Program:DequeueCommand()
 * Returns next command or nil.
 *****************************************************************************/
INTERN int 
Program_DequeueCommand( lua_State* L ) 
{ 
   try { 

		const string cmd = gProgram.DequeueCommand();
		if ( cmd.empty() )
			lua_pushnil( L );
		else
			lua_pushstring( L, cmd.c_str() );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:Exit( int )
 *****************************************************************************/
INTERN int 
Program_Exit( lua_State* L ) 
{ 
   try { 

		EXIT( POP_INT( L ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * milliseconds Program:GetElapsedTime()
 *****************************************************************************/
INTERN int 
Program_GetElapsedTime( lua_State* L ) 
{ 
   try { 

		// Since Lua's tick handler cannot be disable (active while paused),
		// return the elapsed time counted while even paused.
		lua_pushnumber( L, base::Timer::GetInstance().GetElapsedTimeReal() );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * int Program:GetMemUsed()
 *****************************************************************************/
INTERN int 
Program_GetMemUsed( lua_State* L ) 
{ 
   try { 

#if DEBUG
		lua_pushnumber( L, MemUsed() );
#else
		lua_pushnumber( L, 0 );  // N/A in non-DEBUG
#endif
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * string Program:GetScriptDir()
 *****************************************************************************/
INTERN int 
Program_GetScriptDir( lua_State* L ) 
{ 
   try { 

		lua_pushstring( L, GET_LUA().GetScriptDir().c_str() );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * f Program:GetSimulationSpeed()
 *****************************************************************************/
INTERN int 
Program_GetSimulationSpeed( lua_State* L ) 
{ 
   try { 

		lua_pushnumber( L, PhysicsControl::GetSimulationSpeed() );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * milliseconds Program:GetTickFreq()
 * Frequency of program's timer-tick (its pulse).
 *****************************************************************************/
INTERN int 
Program_GetTickFreq( lua_State* L ) 
{ 
   try { 

		lua_pushnumber( L, PROGRAM_CONF.mTimerTickFreq );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * bool Program:IfDebug()
 *****************************************************************************/
INTERN int 
Program_IfDebug( lua_State* L ) 
{ 
   try { 

#if DEBUG
		lua_pushboolean( L, true );
#else
		lua_pushboolean( L, false );
#endif
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * bool Program:IfPaused()
 *****************************************************************************/
INTERN int 
Program_IfPaused( lua_State* L ) 
{ 
   try { 

		lua_pushboolean( L, gProgram.IfPaused() );
		return 1;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:Reset()
 *****************************************************************************/
INTERN int 
Program_Reset( lua_State* L ) 
{ 
   try { 

		gProgram.Reset();
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:SetPause( bool )
 *****************************************************************************/
INTERN int 
Program_SetPause( lua_State* L ) 
{ 
   try { 

		gProgram.SetPause( POP_BOOL( L ) );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Program:SetSimulationSpeed(f)
 *****************************************************************************/
INTERN int 
Program_SetSimulationSpeed( lua_State* L ) 
{ 
   try { 

        const fp simulationSpeed = POP_FP( L );
		PhysicsControl::SetSimulationSpeed( simulationSpeed );
		return 0;  // number of pushed values
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  // new,__gc omitted as Lua cannot instantiate this C++ class.
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  // __gc omitted as Lua cannot instantiate this C++ class.
  { "DequeueCommand", Program_DequeueCommand },
  { "Exit", Program_Exit },
  { "GetElapsedTime", Program_GetElapsedTime },
  { "GetMemUsed", Program_GetMemUsed },
  { "GetScriptDir", Program_GetScriptDir },
  { "GetSimulationSpeed", Program_GetSimulationSpeed },
  { "GetTickFreq", Program_GetTickFreq },
  { "IfDebug", Program_IfDebug },
  { "IfPaused", Program_IfPaused },
  { "Reset", Program_Reset },
  { "SetPause", Program_SetPause },
  { "SetSimulationSpeed", Program_SetSimulationSpeed },
  { NULL, NULL }
};

void
RegisterProgram( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
