/*****************************************************************************
 * $LastChangedDate: 2009-05-18 17:39:53 -0500 (Mon, 18 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	RotationList is a set of matrix rotations to set a Dyna's orientation.
 *//*
 * LEGAL:	COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef OBJECT_ROTATION_LIST_HH
#define OBJECT_ROTATION_LIST_HH 1

#include "math/types_trig.hh"

namespace object {

class RotationList;
class Dyna;

////////////////////////////////////////////////////////////////////////////////
/// @brief RotationList is a set of matrix rotations to set a Dyna's orientation.
///
/// RotationList is useful to set the initial orientation of a 3D model.
/// RotationLists are defined in .conf files (Conf members).
///
/// Example of a rotation list defined in a .conf file:
/// AircraftRotationList  YY -90.0 XX 90.0 YY 8.5
///
class RotationList
{
public:
	/// @brief Defines a rotation (axis and radian).
	///
	class Rotation
	{
	friend class RotationList;
	public:
		Rotation( const uint axis, const Degree degree );
	private:
		uint	mAxis;
		Radian	mRadian;
	};

public:
			RotationList( void );
			~RotationList();
    void    Clear( void ) { mRotationList.clear(); }
	void	Parse( ifstream& strm );
	void	AddRotation( const Rotation& rotation );
	void	Apply( Dyna& dyna ) const;
	void	Apply( Matrix& m ) const;

private:
	list<Rotation>	mRotationList;
};

} // namespace object

#endif // OBJECT_ROTATION_LIST_HH
