/*****************************************************************************
 * $LastChangedDate: 2009-05-23 18:43:35 -0500 (Sat, 23 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define SHADER_MODULE_CC 1
#include "base/module.hh"
using namespace base;
#include "graph/module.hh"
#include "graph/scene_graph.hh"
using namespace graph;
#include "shader/module.hh"
#include "shader/conf.hh"
#include "shader/shader_funcs.hh"
#include "shader/shader_name.hh"
#include "shader/light.hh"

namespace shader {

/*****************************************************************************
 * Initialize module.
 *****************************************************************************/
void
Init( const base::InitArgs& initArgs )
{
	// Load textual conf file into Conf struct.
	const string confTxtFilename = GetConfTxtFilename( initArgs.mProgramName );
	// Not CONF macro since mValid isn't true yet.
	shader::global::gConf.Init( initArgs.mArgc, initArgs.mArgv, confTxtFilename );
}

/*****************************************************************************
 * Start module.
 *****************************************************************************/
void
Start( void )
{
	RefPtr<GroupNode> rootNode = GET_SCENE_GRAPH().GetRootNode();
    Light& light = GET_LIGHT();

    // Shader uniform defaults.
	ResetShaderUniformsForRootNode( *rootNode );

    // Lighting.
    light.SetLightPosition( 0, SHADER_CONF.mLight0Position );
    light.SetLightBrightness( SHADER_CONF.mBrightness );

    // Fog.
	light.SetFogColor( SHADER_CONF.mFogColor );
	light.SetFogDensity( SHADER_CONF.mFogDensity );
}

} // namespace shader
