/*****************************************************************************
 * $LastChangedDate: 2009-05-20 17:31:05 -0500 (Wed, 20 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Shader functions.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define SHADER_SHADER_FUNCS_CC 1
#include "base/module.hh"
using namespace base;
#include "gfx/module.hh"
#include "gfx/osg.hh"
using namespace gfx;
#include "fx/module.hh"
#include "fx/texture.hh"
using namespace fx;
#include "shader/module.hh"
#include "shader/conf.hh"
#include "shader/shader_funcs.hh"
#include "shader/shader_name.hh"

namespace shader {

/*****************************************************************************
 * Disable shader on a node.
 *****************************************************************************/
void
DisableShader( osg::StateSet& stateSet )
{
  //stateSet.removeAttribute( osg::StateAttribute::PROGRAM );  // doesn't work in this case
	stateSet.setAttributeAndModes( new osg::Program, osg::StateAttribute::ON );
}

/*****************************************************************************
 * Reset MOST (not all) shader uniforms.
 *****************************************************************************/
void
ResetShaderUniformsForModel( osg::Node& node )
{
    if ( SHADER_CONF.mEnableShaders )
    {
        // -- see shader_names.hh --
    	SetUniform<bool>(  UNIFORM_NAME_SHADER_MODE_TEXTURE, true,  node );
    	SetUniform<int>(   UNIFORM_NAME_SHADOW_MODE,         0,     node );
    	SetUniform<bool>(  UNIFORM_NAME_GLASS,               false, node );
    	SetUniform<bool>(  UNIFORM_NAME_ALPHA_TEX,           false, node );
    	SetUniform<float>( UNIFORM_NAME_ALPHA_BIAS,          0.0f,  node );
    	SetUniform<int>(   UNIFORM_NAME_TEXTURE_0,           0,     node );  // int, not uint
    	SetUniform<int>(   UNIFORM_NAME_TEXTURE_SHADOW,      fx::defs::TEXTURE_UNIT_SHADOW, node );
    }
}

/*****************************************************************************
 * Reset ALL shader uniforms.
 * Some uniforms are attached only to the root node.
 *****************************************************************************/
void
ResetShaderUniformsForRootNode( osg::Node& node )
{
    if ( SHADER_CONF.mEnableShaders )
    {
        // -- see shader_names.hh --
        ResetShaderUniformsForModel( node );
    	SetUniform<float>(      UNIFORM_NAME_BRIGHTNESS, 1.0f, node );
    	SetUniform<osg::Vec4f>( UNIFORM_NAME_BASE_AMBIENT, osg::Vec4f(0,0,0,0), node );
    }
}

} // namespace shader
