/*****************************************************************************
 * $LastChangedDate: 2009-05-20 15:51:18 -0500 (Wed, 20 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Window class.
 *//*
 * LEGAL:   COPYRIGHT (C) 2007 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef VIEW_WINDOW_HH
#define VIEW_WINDOW_HH 1

#define GET_WINDOW()		(Window::GetInstance())
#define GET_MAIN_VIEW()		(Window::GetInstance().GetMainView())
#define POST_REDISPLAY()	// NOP

#include "gfx/graphics.hh"
using namespace gfx;
#include "view/view.hh"

#include <osg/GraphicsContext>
namespace osg { class Object; class Operation; }
namespace osgGA { class TrackballManipulator; class GUIEventHandler; }
namespace osgViewer { class Viewer; class GraphicsWindow; class StatsHandler; }

namespace view {

class View;

////////////////////////////////////////////////////////////////////////////////
/// @brief Window contains Views, executes main loop, broadcasts draw events.
///
class Window
{
PREVENT_COPYING(Window)
private:
                    		Window( const WidthHeight<int> wh, const string& title );
                            ~Window();
	CLASS_METHOD Window*	CreateInstance( void );

public:
    CLASS_METHOD Window&    GetInstance( void )
	{
		// NewInstance() gets ctor args from conf.
		if ( EX( msInstance != NULL ) )
			return *msInstance;
		else
			return *(msInstance = Window::CreateInstance());

	}

// The main graphics loop:
public:
	void                    EnterMainLoop( void );

// Views:
public:
    void                    AddView( shptr<View> view, const bool mainView = false );
	shptr<View>				GetMainView( void )
							{
							ASSERT( not mViews.empty() );
								return mViews.front();
							}
	void					SaveViewpoint( void );
	void					RestoreViewpoint( void );

// Info:
public:
    WidthHeight<int>        GetWindowSize( void );
	Rect<int>				GetWindowRect( void );
	uint					GetFPS( void ) { return mFPS.ComputeFPS(); }

// OSG (LIMITED USE):
public:
	enum eCamera { eCamera_3D, eCamera_2D };
	void					         OsgAddCamera( RefPtr<osg::Camera> camera, const eCamera cameraType = eCamera_3D );
	RefPtr<osgViewer::GraphicsWindow> OsgGetGraphicsWindow( void );
    void							 OsgSetKeyboardHandler( RefPtr<osgGA::GUIEventHandler> keyboardHandler );
	void							 OsgSetStatsKey( const int key );

private:
	void					ComputeFPS( const Milliseconds elapsedMilsec );
	void					DoOperation( void );
	void					DrawViewOutline( shptr<View> view, const RGBA outlineColor );

private:
	class Operation : public osg::Operation
	{
	public:
	    Operation( void );
		virtual ~Operation();
	    virtual void operator ()( osg::Object* );
	};

private:
	struct FPS
	{
						FPS( void );
		void			Tick( const Milliseconds elapsedMilsec );
		uint			ComputeFPS( void ) { return mFPS; }

		uint			mFPS;			///< FPS of previous second
		uint			mFrameCnt;		///< frame count of current second
		Milliseconds	mFrameStart;	///< beginning of current second
	};

private:
    typedef list<shptr<View> > Views;
    CLASS_VAR Window*						msInstance;			///< there can be only one
	RefPtr<osgViewer::Viewer>				mViewer;			///< contains cameras
    Views									mViews;				///< Window contains one or more Views
	string									mTitle;				///< window title
	FPS										mFPS;				///< frame rate
	RefPtr<osg::GraphicsContext::Traits>	mTraits;
    RefPtr<osg::GraphicsContext>			mGraphicsContext;
    RefPtr<osgViewer::GraphicsWindow>		mGraphicsWindow;
	RefPtr<osg::Operation>					mOperation;
	RefPtr0<osgViewer::StatsHandler>		mStatsHandler;		///< statistics (might be disabled/NULL)
	RefPtr<osgGA::TrackballManipulator>		mTrackballManipulator;
};

} // namespace view

#endif // VIEW_WINDOW_HH
