/*****************************************************************************
 * $LastChangedDate: 2009-05-18 09:11:03 -0500 (Mon, 18 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Test of random functions.
 *//*
 * LEGAL:	COPYRIGHT (C) 2005 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define TESTING 1
#include "test.hh"
#include "base/module.hh"
#include "base/types.hh"
#include "base/random.hh"
using namespace base;

const uint loops = 8*1000*1000;

/*****************************************************************************
 * Test random functions (non-range).
 *****************************************************************************/
bool
TestRandom1( void )
{
	CDEBUG << "Testing random_ui(void) and random_f(void)" << endl;

	GET_RANDOM().randomize( GET_RANDOM().random_ui_true() );

	for ( uint i = 0; i < loops; ++i )
	{
		if ( GET_RANDOM().random_ui() > RAND_MAX )
			return Fail();

		if ( GET_RANDOM().random_f() > 1.0 )
			return Fail();
	}

	return true;
}

/*****************************************************************************
 * Test random functions (range).
 * The goal is to ensure no random value is returned that is greater than
 * the specified range.  Otherwise, since random values are sometimes
 * used as array indexs, the error could cause out-of-bound bugs
 *****************************************************************************/
bool
TestRandomRange( uint range )
{
	CDEBUG << "Testing random_ui(" << range << ") and random_f(" << range << ")" << endl;

	GET_RANDOM().randomize( GET_RANDOM().random_ui_true() );

	for ( uint i = 0; i < loops; ++i )
	{
		uint ui = GET_RANDOM().random_ui( range );
		if ( ui > range )
		{
			CERROR << "ERROR: out-of-range" << endl;
			CERROR << "ui=" << ui << " range=" << range << endl;
			return Fail();
		}

		fp f = GET_RANDOM().random_f( range );
	  //if ( f > range )
		if ( f - 1.0 > range )  // tolerate slight float inaccuracy
		{
			CERROR << "ERROR: out-of-range" << endl;
			CERROR << "f=" << f << " range=" << range << endl;
			return Fail();
		}
	}

	return true;
}

/*****************************************************************************
 * main.
 *****************************************************************************/
int main( int argc, char** argv )
{
	TEST_MAIN_BEGIN;
	int res = 1;
	if (    TestRandom1()
         && TestRandomRange( 1 )
         && TestRandomRange( 2 )
         && TestRandomRange( 30 )
         && TestRandomRange( 1000 ) 
         && TestRandomRange( 0x7fffffff )
         && TestRandomRange( 0xf0000000 ) )
	{
		CDEBUG << "Test PASSED.\n";
		res = 0;
	}
	else
	{
		CDEBUG << "Test FAILED! \n";
		res = 1;
	}
	TEST_MAIN_END;
	exit( res );
}
